-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.Event.Listener = class();

function Wired.Event.Listener:Constructor(event, action, unknown)
  self:SetEvent(event);
  self:SetAction(action);
  self:SetUnknown(unknown);
end

-- getters and setters
function Wired.Event.Listener:GetEvent()
  return self.event;
end

function Wired.Event.Listener:SetEvent(value)
  self.event = value;
end

function Wired.Event.Listener:GetAction()
  return self.action;
end

function Wired.Event.Listener:SetAction(value)
  self.action = value;
end

function Wired.Event.Listener:Unknown()
  return self.unknown;
end

function Wired.Event.Listener:SetUnknown(value)
  self.unknown = value;
end


function Wired.Event.Listener:Execute(sender, args)
  if (type(self:GetAction()) == "function") then
    return self:GetAction()(sender, args);
  end
end

function Wired.Event.Listener:Equals(value)
  return (
    (self:GetEvent() == value:GetEvent()) and
    (self:GetAction() == value:GetAction())
  );
end

-- listener functions

-- get listeners
local GetListeners = function(sender, event)
  if (type(sender.listeners) ~= "table") then
    sender.listeners = {};
  end
  if (type(sender.listeners[event]) ~= "table") then
    sender.listeners[event] = {};
  end
  return sender.listeners[event];
end

-- execute actions for event
local Execute = function(sender, event, args)
  for k, listener in pairs(sender.Wired.Event.Listener:GetListeners(event)) do
    listener:Execute(sender, args);
  end
end

-- add listener
local AddListener = function(sender, value)

  -- localize variables
  local event = value:GetEvent();
  local listeners = sender:GetListeners(event);

  -- get previous action for event
  local previousAction = nil;
  if (not value:Unknown()) then
    previousAction = sender:GetComponent().Event[event];
  else
    previousAction = sender:GetComponent().Wired.Event[event];
  end

  -- check if already linked to event
  local alreadyLinked = false;
  for k, listener in pairs(listeners) do
    alreadyLinked = true;
  end

  -- add listener to the sender's listeners
  table.insert(listeners, value);

  -- add execution link
  if (not alreadyLinked) then
    if (not value:Unknown()) then
      sender:GetComponent().Event[event] = function(sender, args)
        Execute(sender, event, args);
      end
    else
      sender:GetComponent().Wired.Event[event] = function(sender, args)
        Execute(sender:GetComponent(), event, args);
      end
    end
  end

  -- add previous action for event as a listener
  if (previousAction and (not alreadyLinked)) then
    sender:Add(Listener(event, previousAction));
  end
end

-- remove listener
local RemoveListener = function(sender, value)

  -- localize variables
  local event = value:GetEvent();
  local action = value:GetAction();
  local listeners = sender:GetListeners(event);

  -- remove listener from the list
  local anotherLink = false
  for k, listener in pairs(listeners) do
    if ((not action) or value:Equals(listener)) then
      listeners[k] = nil;
    else
      anotherLink = true;
    end
  end

  -- remove execution link
  if (not anotherLink) then
    if (not value:Unknown()) then
      sender:GetComponent().Event[event] = nil;
    else
      sender:GetComponent().Wired.Event[event] = nil;
    end
  end
end

-- add listener context to component
Wired.Event.Listener.Wire = function(context, component)

  -- add context
  if (type(context.Listener) ~= "table") then
    context.Listener = {};
    Wired.Wire(context.Listener, component);

    -- add listener functions
    context.Listener.GetListeners = GetListeners;
    context.Listener.Add = AddListener;
    context.Listener.Remove = RemoveListener;
  end
end
