-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Cell = class();

-- constants
Wired.UI.Cell.Constants = {
  Texture = Wired.UI.Texture("Wired", "UI/Resources/gauge.png")
};

-- defaults
Wired.UI.Cell.Defaults = {
  Title = "Cell",
  Width = 160,
  Height = 20,
  Visible = {
    Base = false,
    TitleLabel = true
  },
  Color = {
    On = Wired.UI.Color.White,
    Off = Wired.UI.Color.Black,
    Base = {
      Disabled = Wired.UI.Color.Grey
    },
    TitleLabel = {
      Disabled = Wired.UI.Color.Magenta
    }
  },
  Active = false,
  Enabled = true
};

function Wired.UI.Cell:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- bar
  self:SetBase(Wired.UI.Trion.Texture(self:GetName() .. "_Cell", self:GetParent()));
  self:GetBase():SetVisible(false);
  self:GetBase():SetTexture(Wired.UI.Cell.Constants.Texture);
  
  -- title label
  self:SetTitleLabel(Wired.UI.Trion.Text(self:GetName() .. "_TitleLabel", self:GetBase()));
  self:GetTitleLabel():SetVisible(false);
  self:GetTitleLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- defaults
  self:ApplyCellDefaults();
  
  -- events
  
  -- left click base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      if (type(self.Wired.Event.LeftClick) == "function") then
        self.Wired.Event:LeftClick();
      end
    end
  ));
  
  -- mouse in base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:SetMouseOver(true);
    end
  ));
  
  -- mosue out base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:SetMouseOver(false);
    end
  ));
end

-- name
function Wired.UI.Cell:GetName()
  return self.name;
end

function Wired.UI.Cell:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Cell:GetParent()
  return self.parent;
end

function Wired.UI.Cell:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Cell:GetBase()
  return self.base;
end

function Wired.UI.Cell:SetBase(value)
  self.base = value;
end

-- title label
function Wired.UI.Cell:GetTitleLabel()
  return self.titleLabel;
end

function Wired.UI.Cell:SetTitleLabel(value)
  self.titleLabel = value;
end

-- list
function Wired.UI.Cell:GetList()
  return self.list;
end

function Wired.UI.Cell:SetList(value)
  self.list = value;
end

-- mouse over
function Wired.UI.Cell:GetMouseOver()
  return self.mouseOver;
end

function Wired.UI.Cell:SetMouseOver(value)
  self.mouseOver = value;

  -- update
  self:Update();
end

-- active
function Wired.UI.Cell:GetActive()
  return self.active;
end

function Wired.UI.Cell:SetActive(value)
  self.active = value;

  -- update
  self:Update();
end

-- enabled
function Wired.UI.Cell:GetEnabled()
  return self.enabled;
end

function Wired.UI.Cell:SetEnabled(value)
  self.enabled = value;

  -- update
  self:Update();
end

-- visible
function Wired.UI.Cell:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.Cell:SetVisible(value)
  self:GetBase():SetVisible(value);
end

-- width
function Wired.UI.Cell:GetWidth()
  return self:GetBase():GetWidth();
end

function Wired.UI.Cell:SetWidth(value)
  self:GetBase():SetWidth(value);
end

-- on color
function Wired.UI.Cell:GetOnColor()
  return self.onColor;
end

function Wired.UI.Cell:SetOnColor(value)
  self.onColor = value;
  
  -- update
  self:Update();
end

-- off color
function Wired.UI.Cell:GetOffColor()
  return self.offColor;
end

function Wired.UI.Cell:SetOffColor(value)
  self.offColor = value;
  
  -- update
  self:Update();
end

-- height
function Wired.UI.Cell:GetHeight()
  return self:GetBase():GetHeight();
end

function Wired.UI.Cell:SetHeight(value)
  self:GetBase():SetHeight(value);
  
  -- update height
  self:UpdateHeight();
end

-- title
function Wired.UI.Cell:GetTitle()
  return self:GetTitleLabel():GetText();
end

function Wired.UI.Cell:SetTitle(value)
  self:GetTitleLabel():SetText(value);
  
  -- update title
  self:UpdateTitle();
end

-- defaults
function Wired.UI.Cell:ApplyCellDefaults()
  
  -- colors
  self:SetOnColor(Wired.UI.Cell.Defaults.Color.On);
  self:SetOffColor(Wired.UI.Cell.Defaults.Color.Off);
  
  -- size
  self:SetWidth(Wired.UI.Cell.Defaults.Width);
  self:SetHeight(Wired.UI.Cell.Defaults.Height);
  
  -- values
  self:SetTitle(Wired.UI.Cell.Defaults.Title);
  
  -- values
  self:SetActive(Wired.UI.Cell.Defaults.Active);
  self:SetEnabled(Wired.UI.Cell.Defaults.Enabled);
  
  -- visibile
  self:GetBase():SetVisible(Wired.UI.Cell.Defaults.Visible.Base);
  self:GetTitleLabel():SetVisible(Wired.UI.Cell.Defaults.Visible.TitleLabel);
end

-- position
function Wired.UI.Cell:SetPosition(left, top)
  self:GetBase():SetPosition(left, top);
end

-- update title
function Wired.UI.Cell:UpdateTitle()
  
  -- width
  self:GetTitleLabel():SetWidth(self:GetTitleLabel():GetFullWidth());
  
  -- update height
  self:UpdateHeight()
end

-- update height
function Wired.UI.Cell:UpdateHeight()
  self:GetTitleLabel():SetHeight(self:GetHeight());
  self:GetTitleLabel():ScaleFontSize();
end

-- update
function Wired.UI.Cell:Update()
  
  -- off
  local color = self:GetOffColor();
  local fontColor = self:GetOnColor();
  
  -- on
  if (self:GetMouseOver() or self:GetActive()) then
    color = self:GetOnColor();
    fontColor = self:GetOffColor();
    
  -- disabled
  elseif (not self:GetEnabled()) then
    color = Wired.UI.Cell.Defaults.Color.Base.Disabled;
    fontColor = Wired.UI.Cell.Defaults.Color.TitleLabel.Disabled;
  end
  
  -- update colors
  self:GetBase():SetColor(color);
  self:GetTitleLabel():SetFontColor(fontColor);
end
