-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.ColorPicker.Picker = class();

-- constants
Wired.UI.ColorPicker.Picker.Constants = {
  Texture = {
    Color = Wired.UI.Texture("Wired", "UI/ColorPicker/Resources/color.png"),
    None = Wired.UI.Texture("Wired", "UI/ColorPicker/Resources/none.png")
  }
};

-- defaults
Wired.UI.ColorPicker.Picker.Defaults = {
  Color = {
    Base = Wired.UI.Color.Grey
  },
  Size = 20,
  Padding = 1,
  Visible = {
    Base = true,
    Box = true
  }
};

function Wired.UI.ColorPicker.Picker:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- base
  self:SetBase(Wired.UI.Trion.Frame(self:GetName() .. "_Base", self:GetParent()));
  self:GetBase():SetVisible(false);
  
  -- box
  self:SetBox(Wired.UI.Trion.Texture(self:GetName() .. "_Box", self:GetBase()));
  self:GetBox():SetVisible(false);
  self:GetBox():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- left click base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:LeftClick();
    end
  ));
  
  -- defaults
  self:ApplyPickerDefaults();
end

-- name
function Wired.UI.ColorPicker.Picker:GetName()
  return self.name;
end

function Wired.UI.ColorPicker.Picker:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.ColorPicker.Picker:GetParent()
  return self.parent;
end

function Wired.UI.ColorPicker.Picker:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.ColorPicker.Picker:GetBase()
  return self.base;
end

function Wired.UI.ColorPicker.Picker:SetBase(value)
  self.base = value;
end

-- box
function Wired.UI.ColorPicker.Picker:GetBox()
  return self.box;
end

function Wired.UI.ColorPicker.Picker:SetBox(value)
  self.box = value;
end

-- padding
function Wired.UI.ColorPicker.Picker:GetPadding()
  if (not self.padding) then
    self.padding = 0;
  end
  return self.padding;
end

function Wired.UI.ColorPicker.Picker:SetPadding(value)
  self.padding = value;
  
  -- update size
  self:UpdateSize();
end

-- size
function Wired.UI.ColorPicker.Picker:GetSize()
  return self.size;
end

function Wired.UI.ColorPicker.Picker:SetSize(value)
  self.size = value;
  
  -- update size
  self:UpdateSize();
end

-- color
function Wired.UI.ColorPicker.Picker:GetColor()
  return self.color;
end

function Wired.UI.ColorPicker.Picker:SetColor(value)
  self.color = value;
  
  -- update
  self:Update();
  
  -- fire color changed event
  if (type(self.Wired.Event.ColorChanged) == "function") then
    self.Wired.Event:ColorChanged();
  end
end

-- visible
function Wired.UI.ColorPicker.Picker:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.ColorPicker.Picker:SetVisible(value)
  self:GetBase():SetVisible(value);
  
  -- fire visible changed event
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- window
function Wired.UI.ColorPicker.Picker:GetWindow()
  return self.window;
end

function Wired.UI.ColorPicker.Picker:SetWindow(value)
  self.window = value;
end

-- defaults
function Wired.UI.ColorPicker.Picker:ApplyPickerDefaults()
  
  -- size
  self:SetSize(Wired.UI.ColorPicker.Picker.Defaults.Size);
  self:SetPadding(Wired.UI.ColorPicker.Picker.Defaults.Padding);
  
  -- visibile
  self:GetBase():SetVisible(Wired.UI.ColorPicker.Picker.Defaults.Visible.Base);
  self:GetBox():SetVisible(Wired.UI.ColorPicker.Picker.Defaults.Visible.Box);
end

-- point
function Wired.UI.ColorPicker.Picker:SetPoint(anchor, offset)
  self:GetBase():SetPoint(anchor, offset);
end

-- position
function Wired.UI.ColorPicker.Picker:SetPosition(x, y)
  self:GetBase():SetPosition(x, y);
end

-- left click
function Wired.UI.ColorPicker.Picker:LeftClick()
  
  -- window
  if (self:GetWindow()) then
    
    -- picker
    self:GetWindow():SetColorPicker(self);
    
    -- color
    self:GetWindow():SetSelected(self:GetColor());
    
    -- show
    self:GetWindow():DisplayAtMouse();
  end
end

-- update size
function Wired.UI.ColorPicker.Picker:UpdateSize()
  
  -- color
  self:GetBase():SetColor(Wired.UI.ColorPicker.Picker.Defaults.Color.Base);
  -- bar
  self:GetBase():SetWidth(self:GetSize());
  self:GetBase():SetHeight(self:GetSize());
  
  -- box
  self:GetBox():SetWidth(self:GetSize() - (self:GetPadding() * 2));
  self:GetBox():SetHeight(self:GetSize() - (self:GetPadding() * 2));
end

-- update
function Wired.UI.ColorPicker.Picker:Update()
  
  -- checked
  if (self:GetColor()) then
    self:GetBox():SetColor(self:GetColor());
    self:GetBox():SetTexture(Wired.UI.ColorPicker.Picker.Constants.Texture.Color);
  else
    self:GetBox():SetColor(Wired.UI.Color.Clear);
    self:GetBox():SetTexture(Wired.UI.ColorPicker.Picker.Constants.Texture.None);
  end
end


--[[
-- imports
import "Turbine.UI";
import "Imez.Event";

Picker = class(Turbine.UI.Control);

-- constants
Picker.Background = "Imez/UI/_Color/Resources/x.tga";
Picker.BlendModes = {
  enabled = Turbine.UI.BlendMode.None,
  disabled = Turbine.UI.BlendMode.Normal
};

-- defaults
Picker.Defaults = {
  None = false,
  Size = {width = 20, height = 20},
};

function Picker:Constructor()
  Turbine.UI.Control.Constructor(self);

  -- window
  self.window = Imez.UI._Color.Window();
  self.window:AddListener(
    Imez.Event.Listener("SelectedColor", function(sender, value)
      self:SetColor(value);

      -- fire selected event
      if (type(self.SelectedColor) == "function") then
        self:SelectedColor(value);
      end

      -- persistence
      self:PromptSave();
    end)
  );

  -- events
  self:AddListener(Imez.Event.Listener("MouseClick", self.Open));

  -- defaults
  self:ApplyDefaults();
end

-- getters and setters
function Picker:IsNoneEnabled()
  return self.window:IsNoneEnabled();
end

function Picker:SetNoneEnabled(value)
  self.window:SetNoneEnabled(value);
end

function Picker:IsNone()
  return self.none;
end

function Picker:SetNone(value)
  self.none = value;

  -- update background visibility
  local mode = Picker.BlendModes.disabled;
  if (value) then
    mode = Picker.BlendModes.enabled;
  end
  self:SetBackColorBlendMode(mode);
end

function Picker:GetColor()
  if (not self:IsNone()) then
    return self:GetBackColor();
  end
end

function Picker:SetColor(value)
  self:SetNone(not value);
  if (value) then
    self:SetBackColor(value);
  end

  -- update window
  self.window:Input(value);
end

-- defaults
function Picker:ApplyDefaults()
  self:SetColor(self.window:GetColor());
  self:SetNone(Picker.Defaults.None);
  self:SetSize(Picker.Defaults.Size.width, Picker.Defaults.Size.height);
  self:SetBackground(Picker.Background);
end

-- open window
function Picker:Open()
  self.window:DisplayAtMouse();
end

-- persistence override
function Picker:Load(data)
  self:SetColor(Turbine.UI.Color:New(data));
  self:SelectedColor(self:GetColor());
end

function Picker:Save()
  if (self:GetColor()) then
    return self:GetColor():Save();
  end
end
--]]