-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Drag = {};

-- functions

-- left down
local LeftDown = function(sender, args)

  -- localization
  local data = sender.Wired.UI.Drag.data;
  if (data.enabled) then

    -- start drag
    data.dragging = true;
    data.dragged = false;

    -- save mouse coordinates
    local mouse = Inspect.Mouse();
    data.x = mouse.x - sender:GetLeft();
    data.y = mouse.y - sender:GetTop();
  end
end

-- left up
local LeftUp = function(sender, args)

  -- localization
  local data = sender.Wired.UI.Drag.data;
  if (data.enabled) then

    -- stop drag
    data.dragging = false;
  end
end

-- mouse move
local MouseMove = function(sender, args)

  -- localization
  local data = sender.Wired.UI.Drag.data;
  if (data.enabled) then
    if (data.dragging) then
      data.dragged = true;

      -- apply mouse coordinates
      local mouse = Inspect.Mouse();
      sender:SetPoint(
        "TOPLEFT",
        sender:GetParent(),
        "TOPLEFT",
        mouse.x - data.x,
        mouse.y - data.y
      );
    end
  end
end

-- override LeftClick to differentiate between drag and click
local LeftClick = function(sender, args)
  if ((not sender.Wired.UI.Drag.data.enabled) or (not sender.Wired.UI.Drag.data.dragged)) then
    if (type(sender.Wired.Event.StationaryClick) == "function") then
      sender.Wired.Event:StationaryClick(args);
    end
  end
end

-- listeners
Wired.UI.Drag.Listeners = {
  Wired.Event.Listener("LeftDown", LeftDown),
  Wired.Event.Listener("LeftUp", LeftUp),
  Wired.Event.Listener("MouseMove", MouseMove),
  Wired.Event.Listener("LeftClick", LeftClick)
};

-- enable
local Enable = function(sender, value)

  -- add data
  if (not sender.data) then
    sender.data = {};
  end

  -- reset data
  if (value) then
    sender.data.enabled = true;
    sender.data.dragging = false;
    sender.data.dragged = false;
    sender.data.x = 0;
    sender.data.y = 0;
  else
    sender.data.enabled = false;
  end

  -- reset listeners
  for k, listener in pairs(Wired.UI.Drag.Listeners) do
    if (value) then
      sender:GetComponent().Wired.Event.Listener:Add(listener);
    else
      sender:GetComponent().Wired.Event.Listener:Remove(listener);
    end
  end
end

-- add drag context to component
Wired.UI.Drag.Wire = function(context, component)

  -- add context
  if (type(context.Drag) ~= "table") then
    context.Drag = {};
    Wired.Wire(context.Drag, component);

    -- add drag functions
    context.Drag.Enable = Enable;
  end
end
