-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Gauge = class();

-- constants
Wired.UI.Gauge.Constants = {
  Texture = Wired.UI.Texture("Wired", "UI/Resources/gauge.png"),
  Size = {
    Min = 3,
    Max = 100,
    Ratio = 12
  },
  Fade = {
    Min = 0,
    Max = 1,
    Duration = 2
  }
};

-- defaults
Wired.UI.Gauge.Defaults = {
  Colors = {
    Base = Wired.UI.Color.Black,
    Font = Wired.UI.Color.White
  },
  Percentage = true,
  Title = "Gauge Title",
  Size = 25,
  Visible = {
    Base = false,
    Bar = true,
    TitleLabel = true,
    ValueLabel = true,
    MarkLabel = true
  },
  Scale = 100,
  Inverted = false,
  Flipped = false
};

function Wired.UI.Gauge:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- base
  self:SetBase(Wired.UI.Trion.Frame(self:GetName() .. "_Base", self:GetParent()));
  self:GetBase():SetVisible(false);
  
  -- bar
  self:SetBar(Wired.UI.Trion.Texture(self:GetName() .. "_Bar", self:GetBase()));
  self:GetBar():SetVisible(false);
  self:GetBar():SetTexture(Wired.UI.Gauge.Constants.Texture);
  self:GetBar():SetLayer(0);
  self:GetBar():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 0, y = 0});
  
  -- title label
  self:SetTitleLabel(Wired.UI.TextShadow(self:GetName() .. "_TitleLabel", self:GetBase()));
  self:GetTitleLabel():SetVisible(false);
  self:GetTitleLabel():SetLayer(1);
  self:GetTitleLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- value label
  self:SetValueLabel(Wired.UI.TextShadow(self:GetName() .. "_ValueLabel", self:GetBase()));
  self:GetValueLabel():SetVisible(false);
  self:GetValueLabel():SetLayer(1);
  self:GetValueLabel():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = 0, y = 0});
  
  -- mark label
  self:SetMarkLabel(Wired.UI.TextShadow(self:GetName() .. "_MarkLabel", self:GetBase()));
  self:GetMarkLabel():SetVisible(false);
  self:GetMarkLabel():SetLayer(1);
  self:GetMarkLabel():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 0, y = 0});
  
  -- apply defaults
  self:ApplyGaugeDefaults();
end

-- name
function Wired.UI.Gauge:GetName()
  return self.name;
end

function Wired.UI.Gauge:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Gauge:GetParent()
  return self.parent;
end

function Wired.UI.Gauge:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Gauge:GetBase()
  return self.base;
end

function Wired.UI.Gauge:SetBase(value)
  self.base = value;
end

-- bar
function Wired.UI.Gauge:GetBar()
  return self.bar;
end

function Wired.UI.Gauge:SetBar(value)
  self.bar = value;
end

-- title label
function Wired.UI.Gauge:GetTitleLabel()
  return self.titleLabel;
end

function Wired.UI.Gauge:SetTitleLabel(value)
  self.titleLabel = value;
end

-- value label
function Wired.UI.Gauge:GetValueLabel()
  return self.valueLabel;
end

function Wired.UI.Gauge:SetValueLabel(value)
  self.valueLabel = value;
end

-- mark label
function Wired.UI.Gauge:GetMarkLabel()
  return self.markLabel;
end

function Wired.UI.Gauge:SetMarkLabel(value)
  self.markLabel = value;
end

-- title
function Wired.UI.Gauge:GetTitle()
  return self:GetTitleLabel():GetText();
end

function Wired.UI.Gauge:SetTitle(value)
  self:GetTitleLabel():SetText(value);
end

-- percentage
function Wired.UI.Gauge:GetPercentage()
  return self.percentage;
end

function Wired.UI.Gauge:SetPercentage(value)
  self.percentage = value;
end

-- size
function Wired.UI.Gauge:GetSize()
  return self.size;
end

function Wired.UI.Gauge:SetSize(value)

  -- restrict
  self.size = math.min(
    Wired.UI.Gauge.Constants.Size.Max,
    math.max(Wired.UI.Gauge.Constants.Size.Min, value)
  );
  
  -- update size
  self:UpdateSize();
end

-- scale
function Wired.UI.Gauge:GetScale()
  return self.scale;
end

function Wired.UI.Gauge:SetScale(value)
  self.scale = value;
end

-- value
function Wired.UI.Gauge:GetValue()
  return self.value;
end

function Wired.UI.Gauge:SetValue(value)

  -- value changed
  if (value ~= self.value) then
    self.value = value;
  
    -- draw
    self:Draw();
  
    -- fire value changed event
    if (type(self.Wired.Event.ValueChanged) == "function") then
      self.Wired.Event:ValueChanged();
    end
  end
end

-- mark
function Wired.UI.Gauge:GetMark()
  return self.mark;
end

function Wired.UI.Gauge:SetMark(value)

  -- value changed
  if (value ~= self.mark) then
    self.mark = value;
  
    -- draw
    self:Draw();
  
    -- fire mark changed event
    if (type(self.Wired.Event.MarkChanged) == "function") then
      self.Wired.Event:MarkChanged();
    end
  end
end

-- visible
function Wired.UI.Gauge:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.Gauge:SetVisible(value)
  self:GetBase():SetVisible(value);
  
  -- fire visible changed event
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- inverted
function Wired.UI.Gauge:GetInverted()
  return self.inverted
end

function Wired.UI.Gauge:SetInverted(value)
  self.inverted = value;
end

-- flipped
function Wired.UI.Gauge:GetFlipped()
  return self.flipped
end

function Wired.UI.Gauge:SetFlipped(value)
  self.flipped = value;
end

-- color
function Wired.UI.Gauge:GetColor()
  return self.color;
end

function Wired.UI.Gauge:SetColor(value)
  self.color = value;
  
  -- update bar
  self:GetBar():SetColor(value);
end

-- font color
function Wired.UI.Gauge:GetFontColor()
  return self.fontColor;
end

function Wired.UI.Gauge:SetFontColor(value)
  self.fontColor = value;
  
  -- update font color
  self:UpdateFontColor();
end

function Wired.UI.Gauge:UpdateFontColor()
  self:GetTitleLabel():SetFontColor(self:GetFontColor());
  self:GetValueLabel():SetFontColor(self:GetFontColor());
  self:GetMarkLabel():SetFontColor(self:GetFontColor());
end

-- width
function Wired.UI.Gauge:GetWidth()
  return self:GetBase():GetWidth();
end

-- static width
function Wired.UI.Gauge:GetStaticWidth()
  return self.staticWidth;
end

function Wired.UI.Gauge:SetStaticWidth(value)
  self.staticWidth = value;
  
  -- update size
  self:UpdateSize();
end

-- fade
function Wired.UI.Gauge:GetFade()
  return self.fade
end

function Wired.UI.Gauge:SetFade(value)
  self.fade = value;
end

-- defaults
function Wired.UI.Gauge:ApplyGaugeDefaults()
  
  -- base color
  self:GetBase():SetColor(Wired.UI.Gauge.Defaults.Colors.Base);
  
  -- font colors
  self:SetFontColor(Wired.UI.Gauge.Defaults.Colors.Font);
  
  -- values
  self:SetPercentage(Wired.UI.Gauge.Defaults.Percentage);
  self:SetTitle(Wired.UI.Gauge.Defaults.Title);
  self:SetInverted(Wired.UI.Gauge.Defaults.Inverted);
  self:SetFlipped(Wired.UI.Gauge.Defaults.Flipped);
  
  -- reset
  self:Reset(self:GetTitle(), Wired.UI.Gauge.Defaults.Scale);
  self:SetSize(Wired.UI.Gauge.Defaults.Size);
  
  -- visibile
  self:GetBase():SetVisible(Wired.UI.Gauge.Defaults.Visible.Base);
  self:GetBar():SetVisible(Wired.UI.Gauge.Defaults.Visible.Bar);
  self:GetTitleLabel():SetVisible(Wired.UI.Gauge.Defaults.Visible.TitleLabel);
  self:GetValueLabel():SetVisible(Wired.UI.Gauge.Defaults.Visible.ValueLabel);
  self:GetMarkLabel():SetVisible(Wired.UI.Gauge.Defaults.Visible.MarkLabel);
end

-- position
function Wired.UI.Gauge:SetPosition(x, y)
  self:GetBase():SetPosition(x, y);
end

-- set point
function Wired.UI.Gauge:SetPoint(anchor, offset)
  self:GetBase():SetPoint(anchor, offset);
end

-- scale color
function Wired.UI.Gauge:GetScaledRed()
  local scale = self:GetScale() / 2;
  return 1 - ((math.max(scale, self:GetValue()) - scale) / scale);
end

function Wired.UI.Gauge:GetScaledGreen()
  local scale = self:GetScale() / 2;
  return math.max(0, math.min(scale, self:GetValue()) / scale);
end

-- scale color inverted
function Wired.UI.Gauge:GetScaledRedInverted()
  local scale = self:GetScale() / 2;
  return 1 - ((math.max(scale, ((scale * 2) - self:GetValue())) - scale) / scale);
end

function Wired.UI.Gauge:GetScaledGreenInverted()
  local scale = self:GetScale() / 2;
  return math.max(0, math.min(scale, ((scale * 2) - self:GetValue())) / scale);
end

function Wired.UI.Gauge:ScaleColor()
  if (self:GetInverted()) then
    self:GetBar():SetColor(Wired.UI.Color(
      
      self:GetScaledGreenInverted(),
      self:GetScaledRedInverted(),
      0,
      1
    ));
  elseif (self:GetFlipped()) then
    self:GetBar():SetColor(Wired.UI.Color(
      self:GetScaledGreen(),
      self:GetScaledRed(),
      0,
      1
    ));
  else
    self:GetBar():SetColor(Wired.UI.Color(
      self:GetScaledRed(),
      self:GetScaledGreen(),
      0,
      1
    ));
  end
end

-- scale width
function Wired.UI.Gauge:GetScaledBar()
  local result = self:GetBase():GetWidth();
  if (self:GetScale()) then
    result = math.max(0, math.min(result,
      ((self:GetValue() / self:GetScale()) * result)
    ));
  end
  if (self:GetFlipped()) then
    result = self:GetBase():GetWidth() - result;
  end
  return result;
end

function Wired.UI.Gauge:ScaleBar()
  self:GetBar():SetWidth(self:GetScaledBar());
end

-- scale value
function Wired.UI.Gauge:GetScaledValue()
  local result = 0;
  if (self:GetPercentage()) then
    result = math.ceil(
      (self:GetValue() * 100) / self:GetScale()
    );
  else
    result = math.ceil(math.max(0, math.min(
      self:GetScale(),
      self:GetValue()
    )));
  end
  return result;
end

function Wired.UI.Gauge:ScaleValue()
  local count = self:GetScaledValue();
  if (count ~= self:GetValueLabel():GetText()) then
    self:GetValueLabel():SetText(tostring(count));
    self:GetValueLabel():SetWidth(self:GetValueLabel():GetFullWidth());
  end
end

-- update mark
function Wired.UI.Gauge:UpdateMark()
  local mark = "";
  if (self:GetMark()) then
    mark = self:GetMark();
  end
  if (mark ~= self:GetMarkLabel():GetText()) then
    self:GetMarkLabel():SetText(mark);
    self:GetMarkLabel():SetWidth(self:GetMarkLabel():GetFullWidth());
  end
end

-- draw
function Wired.UI.Gauge:Draw()

  -- scale color
  if (not self:GetColor()) then
    self:ScaleColor();
  end
  
  -- scale width
  self:ScaleBar();
  
  -- scale value
  self:ScaleValue();
  
  -- update mark
  self:UpdateMark();
end

-- reset
function Wired.UI.Gauge:Reset(title, scale, value)
  
  -- title
  self:SetTitle(title);
  
  -- scale
  if (scale) then
    self:SetScale(scale);
  else
    self:SetScale(1);
  end
  if (value) then
    self:SetValue(value);
  else
    self:SetValue(self:GetScale());
  end
  
  -- fix this circle
  
  -- draw
  if (self:GetSize()) then
    self:UpdateSize();
  else
    self:Draw();
  end
  
  -- fix this circle
  
  -- show
  if (scale) then
    self:SetVisible(true);
  else
    self:SetVisible(false);
  end
end

-- update size
function Wired.UI.Gauge:UpdateSize()
  
  -- size components
  self:GetBase():SetHeight(self:GetSize());
  self:GetBar():SetHeight(self:GetSize());
  
  -- size labels
  self:GetTitleLabel():SetHeight(self:GetSize());
  self:GetValueLabel():SetHeight(self:GetSize());
  self:GetMarkLabel():SetHeight(self:GetSize());
  
  -- init title label font size
  self:GetTitleLabel():SetFontSize(self:GetSize());
  
  -- decrease title label font size to fit
  local shrink = true;
  local lastSize = -1;
  while (shrink) do
    local size = self:GetTitleLabel():GetFullHeight();
    if ((lastSize ~= size) and (size > self:GetSize())) then
      lastSize = size;
      self:GetTitleLabel():SetFontSize(self:GetTitleLabel():GetFontSize() - 1);
    else
      shrink = false;
    end
  end
  
  -- resize other labels
  self:GetValueLabel():SetFontSize(self:GetTitleLabel():GetFontSize());
  self:GetMarkLabel():SetFontSize(self:GetTitleLabel():GetFontSize());
  
  -- resize label width
  self:GetTitleLabel():SetWidth(self:GetTitleLabel():GetFullWidth());
  self:GetValueLabel():SetWidth(self:GetValueLabel():GetFullWidth());
  self:GetMarkLabel():SetWidth(self:GetMarkLabel():GetFullWidth());

  -- resize base
  if (not self:GetStaticWidth()) then
    self:GetBase():SetWidth(self:GetSize() * Wired.UI.Gauge.Constants.Size.Ratio);
  else
    self:GetBase():SetWidth(self:GetStaticWidth());
  end
  
  -- draw
  self:Draw();
end

-- fade
function Wired.UI.Gauge:Fade(idle)
  
  -- reset
  if (not self:GetFade()) then
    self:SetFade({
      alpha = self:GetBase():GetAlpha(),
      value = 1
    });
    
  -- decrease
  else
    self:GetFade().value = math.max(0, (
      self:GetFade().value -
      (idle:GetElapsed() / Wired.UI.Gauge.Constants.Fade.Duration)
    ));
  end
  
  -- alpha
  self:GetBase():SetAlpha(self:GetFade().value);
  
  -- color
  if (self:GetColor() ~= Wired.UI.Color.Gauge.Interrupted) then
    self:GetFade().color = self:GetColor();
    self:SetColor(Wired.UI.Color.Gauge.Interrupted);
  end
  
  -- stop
  if (self:GetFade().value <= 0) then
    self:StopFade();
  end
end

function Wired.UI.Gauge:StopFade()
  self:GetBase():SetAlpha(self:GetFade().alpha);
  self:SetColor(self:GetFade().color);
  self:SetFade();
  self:SetVisible(false);
end
