-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Trion.Text = class(Wired.UI.Trion.Base);

-- defaults
Wired.UI.Trion.Text.Defaults = {
  Colors = {
    Background = Wired.UI.Color.Clear,
    Font = Wired.UI.Color.Grey
  }
};

function Wired.UI.Trion.Text:Constructor(name, parent)
  Wired.UI.Trion.Base.Constructor(self, name, parent);
  
  -- create base
  self:SetBase(UI.CreateFrame(
    Wired.UI.Trion.Base.Constants.Types.Text, self:GetName(), self:GetParent():GetBase())
  );
  
  -- defaults
  self:ApplyBaseDefaults();
  self:ApplyTextDefaults();
end

-- text
function Wired.UI.Trion.Text:GetText()
  return self:GetBase():GetText();
end

function Wired.UI.Trion.Text:SetText(value)
  self:GetBase():SetText(value);
end

-- font color
function Wired.UI.Trion.Text:GetFontColor()
  return self.fontColor;
end

function Wired.UI.Trion.Text:SetFontColor(value)
  self.fontColor = value;
  
  -- update base
  if (value:GetAlpha()) then
    self:GetBase():SetFontColor(value:GetRed(), value:GetGreen(), value:GetBlue(), value:GetAlpha());
  else
    self:GetBase():SetFontColor(value:GetRed(), value:GetGreen(), value:GetBlue());
  end
end

-- font size
function Wired.UI.Trion.Text:GetFontSize()
  return self:GetBase():GetFontSize();
end

function Wired.UI.Trion.Text:SetFontSize(value)
  self:GetBase():SetFontSize(value);
end

-- metrics
function Wired.UI.Trion.Text:GetFullWidth()
  return self:GetBase():GetFullWidth();
end

function Wired.UI.Trion.Text:GetFullHeight()
  return self:GetBase():GetFullHeight();
end

-- wrap
function Wired.UI.Trion.Text:GetWrapping()
  return self:GetBase():GetWordwrap();
end

function Wired.UI.Trion.Text:SetWrapping(value)
  self:GetBase():SetWordwrap(value);
end

-- defaults
function Wired.UI.Trion.Text:ApplyTextDefaults()
  self:SetColor(Wired.UI.Trion.Text.Defaults.Colors.Background);
  self:SetFontColor(Wired.UI.Trion.Text.Defaults.Colors.Font);
end

-- scale font size
function Wired.UI.Trion.Text:ScaleFontSize()
  
  -- init font size
  self:SetFontSize(self:GetHeight());
  
  -- decrease font size to fit
  local shrink = true;
  local lastSize = -1;
  while (shrink) do
    local size = self:GetFullHeight();
    if ((lastSize ~= size) and (size > self:GetHeight())) then
      lastSize = size;
      self:SetFontSize(self:GetFontSize() - 1);
    else
      shrink = false;
    end
  end
end
