-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.Utility.CommandManager = class();

function Wired.Utility.CommandManager:Constructor(application, slash)

  -- wire
  Wire(self);
  
  -- parameters
  self:SetApplication(application);
  self:SetSlash(slash);
  
  -- events
  table.insert(
    Command.Slash.Register(self:GetSlash()),
    {
      function(value)
        self:Parse(value);
      end,
      self:GetApplication():GetName(),
      self:GetApplication():GetName() .. " Command Manager"
    }
  );
end

-- application
function Wired.Utility.CommandManager:GetApplication()
  return self.application;
end

function Wired.Utility.CommandManager:SetApplication(value)
  self.application = value;
end

-- slash
function Wired.Utility.CommandManager:GetSlash()
  return self.slash;
end

function Wired.Utility.CommandManager:SetSlash(value)
  self.slash = value;
end

-- commands
function Wired.Utility.CommandManager:GetCommands()
  if (type(self.commands) ~= "table") then
    self.commands = {};
  end
  return self.commands;
end

function Wired.Utility.CommandManager:SetCommands(value)
  self.commands = value;
end

function Wired.Utility.CommandManager:AddCommand(value)
  table.insert(self:GetCommands(), value);
end

-- parse
function Wired.Utility.CommandManager:Parse(value)
  
  -- uppercase
  local head = string.upper(value);
  
  -- get number
  local number = 0;
  if (value:find(" ")) then
    number = tonumber(value:sub((value:find(" ") + 1)));
  end
  
  local found = false;
  for k, command in pairs(self:GetCommands()) do
    if (head:find(string.upper(command:GetCommand()))) then
      command:Execute(number);
      found = true;
    end
  end
  
  -- execute default command
  if (not found) then
    self:ListCommands();
  end
end

-- list commands
function Wired.Utility.CommandManager:ListCommands()
  print(self:GetApplication():GetName() .. " Commands:");
  for k, command in pairs(self:GetCommands()) do
    print(
      "/" ..
      self:GetSlash() .. " " ..
      command:GetCommand() .. " [" ..
      command:GetDescription() .. "]"
    );
  end
end
